//
//  Player.cpp
//  papTermOneCourseWork
//
//  Created by Charles Ringer on 28/10/2015.
//
//

#include "Player.hpp"

Player::Player(vector<Wall> &walls, ImageBank &imgBank, vector<Enemy> &badGuys): height(20), width(20), speed(2)
{
    enemies = &badGuys;
    //This finds a random place on the map that is not occupied by a wall
    bool done = false;
    //Potentail X and Y
    int potX, potY;
    while(!done)
    {
        //Create a new potential x and y
        potX = 20*((int)ofRandom(1, 39));
        potY = 20*((int)ofRandom(1, 29));
        //If it does not hit any of the walls
        if (!(std::any_of(walls.begin(), walls.end(),
                          [potX, potY](Wall wall)
            {
                return potY < wall.getEndY() &&
                potY+20 >= wall.getStartY() &&
                potX < wall.getEndX() &&
                potX+20 > wall.getStartX();
            })))
        {
            //Set the X and Y vals to these values
            X = potX;
            Y = potY;
            //And get out of this loop
            done = true;
        }
    }
    
    //Loads the player images
    for (int i = 0; i < 4; i++)
    {
        string filePath = "Assets/Player/Walking/" + std::to_string(i) + ".png";
        walkingBot.push_back(imgBank.loadImage(filePath));
    }
    
    standingBot = imgBank.loadImage("Assets/Player/Standing/0.png");
    top = imgBank.loadImage("Assets/Player/Upper/0.png");
    torchTop = imgBank.loadImage("Assets/Player/Upper/2.png");

}

void Player::disp()
{
    ofSetColor(255);
    if(goingUp||goingDown||goingLeft||goingRight)
    {
        ofImage tempBot = *walkingBot[currentBottom];
        if (goingRight) tempBot.rotate90(1);
        if (goingDown) tempBot.rotate90(2);
        if (goingLeft) tempBot.rotate90(3);
        tempBot.draw(X,Y);
        if (ofGetFrameNum() %5==0)
        {
            if (currentBottom <3) currentBottom++;
            else currentBottom = 0;
        }
        ofImage tempTop;
        if (torchOn) tempTop.clone(*torchTop);
        else tempTop.clone(*top);
        prevRot = 0;
        if (goingRight)
        {
            tempTop.rotate90(1);
            prevRot = 1;
        }
        else if (goingDown)
        {
            tempTop.rotate90(2);
            prevRot = 2;
        }
        else if (goingLeft)
        {
            tempTop.rotate90(3);
            prevRot = 3;
        }
        tempTop.draw(X,Y);
    }
        else
    {
        standingBot->draw(X,Y);
        ofImage tempTop;
        if (torchOn) (tempTop).clone(*torchTop);
        else tempTop.clone(*top);
        if (prevRot==1) tempTop.rotate90(1);
        if (prevRot==2) tempTop.rotate90(2);
        if (prevRot==3) tempTop.rotate90(3);
        tempTop.draw(X,Y);
    }
    
}

void Player::move(vector<Wall> &walls)
{
    if (up && canMove('u', walls)){ goingUp = true; Y-=speed;}
    else goingUp = false;
    
    if (down && canMove('d', walls)){goingDown = true; Y+=speed; }
    else goingDown = false;

    if (left && canMove('l', walls)){goingLeft = true;X-=speed;}
    else goingLeft = false;
    
    if (right && canMove('r', walls)){ goingRight = true; X+=speed; }
    else goingRight = false;
    
}

bool Player::canMove(char dir, vector<Wall> &walls)
{
    if (dir == 'u' && std::any_of(walls.begin(), walls.end(), [this](Wall wall){return Y-speed < wall.getEndY() && Y >= wall.getEndY() && X < wall.getEndX() && X+width > wall.getStartX();}))return false;
    else if (dir == 'd' && std::any_of(walls.begin(), walls.end(), [this](Wall wall){return Y+height+speed > wall.getStartY() && Y+height <= wall.getStartY() && X < wall.getEndX() && X+width > wall.getStartX();})) return false;
    else if (dir == 'l' && std::any_of(walls.begin(), walls.end(), [this](Wall wall){return X-speed < wall.getEndX() && X >= wall.getEndX() && Y < wall.getEndY() && Y+width > wall.getStartY();})) return false;
    else if (dir == 'r' && std::any_of(walls.begin(), walls.end(), [this](Wall wall){return X+width+speed > wall.getStartX() && X+width <= wall.getStartX() && Y < wall.getEndY() && Y+width > wall.getStartY();})) return false;
    return true;
}



void Player::handleKeyOn(int keyIn)
{
    if (keyIn == 'w') up = true;
    else if (keyIn == 'a') left = true;
    else if (keyIn == 's') down = true;
    else if (keyIn == 'd') right = true;
    else if (keyIn == 'e') torchOn = !torchOn;
    //else if (keyIn == ' ') attack(enemies);
}

void Player::handleKeyOff(int keyIn)
{
    if (keyIn == 'w') up = false;
    else if (keyIn == 'a') left = false;
    else if (keyIn == 's') down = false;
    else if (keyIn == 'd') right = false;
}

int Player::checkGameEnd(Endpoint *endPoint)
{
    if((X+width) > endPoint->getSX() && X < endPoint->getEX()
       && (Y+width) > endPoint->getSY() && Y < endPoint->getEY()) return 1;
    
    if(std::any_of(enemies->begin(), enemies->end(), [this](Enemy enemy)
    {return (X+width) > (enemy.getSX()+10) && X < (enemy.getEX()-10)
    && (Y+width) > (enemy.getSY()+10) && Y < (enemy.getEY()-10);}))return -1;
    
    return 0;
}

bool Player::attack()
{
    return true;
}